import argparse
import csv
import logging
import numpy as np
import pandas as pd 
from sklearn import linear_model
import apache_beam as beam
from apache_beam.metrics.metric import Metrics
from apache_beam.options.pipeline_options import PipelineOptions
from apache_beam.options.pipeline_options import SetupOptions

class Prediction(beam.DoFn):
    def process(self, element):
        Date,Open,High,Low,Close = element.split(",")
        x_predict = [float(Open),float(High),float(Low)]
        x_predict = (np.asarray(x_predict)).reshape(1,-1)
        predictions = regr.predict(x_predict)
        return [{
            'Date':Date,
            'Open': float(Open),
            'High': float(High),
            'Low' : float(Low),
            'Close': float(Close),
            'Close_predict': predictions[0]
        }]
    
class CollectOpen(beam.DoFn):
    def process(self, element):
        # Returns a list of tuples containing Date and Open value
        result = [(element['Date'],element['Open'],element['High'],element['Low'],element['Close'],element['Close_predict'])]
        return result


def run(argv=None):
    stock = pd.read_csv("E:\\phd_data\\apache-beam-2.4.0\\sdks\\python\\apache_beam\\examples\\complete\\Stock_Prediction\\Final\\stock_train_without_parameters.csv")  
    #import gcsfs

	# Cloud CSV Read Code
    #fs=gcsfs.GCSFileSystem(project='newproj-228110')
    #with fs.open('stock_prediction/stock_train_w_parameters.csv') as f:
        #stock = pd.read_csv(f)
    
    global regr
    regr = linear_model.LinearRegression()
    array = stock.values
    X = array[:,1:4]
    Y = array[:,4]

    regr.fit(X, Y)

    parser = argparse.ArgumentParser()
    parser.add_argument('--input',type=str,
                        help='Input for the pipeline',
                        default='stock_test.csv')
    parser.add_argument('--output',
                        help='Output for the pipeline',
                        default='NHB.txt')
    parser.add_argument('--setup_file',
                        help='Output for the pipeline',
                        default='E:\\phd_data\\apache-beam-2.4.0\\sdks\\python\\apache_beam\\examples\\complete\\Stock_Prediction\\cloud\\With parameter\\stock_prediction_w_parameters_cloud.py')
   
    args, pipeline_args = parser.parse_known_args(argv)
    options = PipelineOptions(pipeline_args)
    options.view_as(SetupOptions).save_main_session = True
    input_filename = args.input
    output_filename = args.output
    with beam.Pipeline(options=options) as p:
        (p
         | 'ReadInputText' >> beam.io.ReadFromText(args.input,skip_header_lines=1)
         | 'Prediction' >> beam.ParDo(Prediction())
         | beam.ParDo(CollectOpen())
         | beam.io.textio.WriteToText(args.output))

if __name__ == '__main__':
    logging.getLogger().setLevel(logging.INFO)
    run()
