import argparse
import csv
import logging
import numpy as np
import pandas as pd 
from sklearn import linear_model
import apache_beam as beam
from apache_beam.metrics.metric import Metrics
from apache_beam.options.pipeline_options import PipelineOptions
from apache_beam.options.pipeline_options import SetupOptions
from matplotlib import pyplot as plt

class Split(beam.DoFn):
    def process(self, element):
        Date,Open,High,Low,Gold,Close = element.split(",")
        x_predict = [float(Open),float(High),float(Low),float(Gold)]
        x_axis.append(float(Close))
        open_axis.append(float(Open))
        x_predict = (np.asarray(x_predict)).reshape(1,-1)
        predictions = regr.predict(x_predict)
        y_axis.append(round(predictions[0], 2))
        return [{
            'Date':Date,
            'Open': float(Open),
            'Gold' : float(Gold),
            'Close': float(Close),
            'Close_predict': round(predictions[0], 2)
        }]
    
class CollectOpen(beam.DoFn):
    def process(self, element):
        # Returns a list of tuples containing Date and Open value
        result = [(element['Date'],element['Open'],element['Gold'],element['Close'],element['Close_predict'])]
        return result


def run(argv=None):
    stock = pd.read_csv("E:\\phd_data\\apache-beam-2.4.0\\sdks\\python\\apache_beam\\examples\\complete\\Stock_Prediction\\stock_train_with_gold.csv")  
   # import gcsfs
    import matplotlib
    from matplotlib import pyplot as plt
	
	# Cloud CSV Read Code
   # fs=gcsfs.GCSFileSystem(project='newproj-228110')
   # with fs.open('stock_prediction/train.csv') as f:
   #    stock = pd.read_csv(f)
    
    global regr
    global x_axis, y_axis, open_axis
    x_axis = []
    y_axis = []
    open_axis = []
    regr = linear_model.LinearRegression()
    array = stock.values
    X = array[:,1:5]
    Y = array[:,5]

    regr.fit(X, Y)

    parser = argparse.ArgumentParser()
    parser.add_argument('--input',type=str,
                        help='Input for the pipeline',
                        default='stock_test_with_gold.csv')
    parser.add_argument('--output',
                        help='Output for the pipeline',
                        default='NHB.txt')
    parser.add_argument('--setup_file',
                        help='Output for the pipeline',
                        default='E:\\phd_data\\apache-beam-2.4.0\\sdks\\python\\apache_beam\\examples\\complete\\Stock_Prediction\\stock_prediction_gold.py')
   
    args, pipeline_args = parser.parse_known_args(argv)
    options = PipelineOptions(pipeline_args)
    options.view_as(SetupOptions).save_main_session = True
    input_filename = args.input
    output_filename = args.output
    with beam.Pipeline(options=options) as p:
        (p
         | 'ReadInputText' >> beam.io.ReadFromText(args.input,skip_header_lines=1)
         | 'Split' >> beam.ParDo(Split())
         | beam.ParDo(CollectOpen())
         | beam.io.textio.WriteToText(args.output))
    from matplotlib import pyplot as plt
    fit = np.polyfit(x_axis,y_axis,1)
    fit_fn = np.poly1d(fit)
    area = np.pi*2
    plt.xlabel('Close Value')
    plt.ylabel('Open Value')
    plt.title('Stock Predicion')
    plt.plot(y_axis,open_axis, color="green")
    plt.scatter(y_axis, open_axis, s=area, c="green", alpha=0.5)
    plt.scatter(x_axis, open_axis, s=area, c="red", alpha=0.5)
    plt.savefig('result_gold.png')

if __name__ == '__main__':
    logging.getLogger().setLevel(logging.INFO)
    run()
