import argparse
import csv
import logging

import apache_beam as beam
from apache_beam.metrics.metric import Metrics
from apache_beam.options.pipeline_options import PipelineOptions
from apache_beam.options.pipeline_options import SetupOptions

class Split(beam.DoFn):
    def process(self, element):
        Date,Open,High,Low,Close,Volume = element.split(",")
        return [{
            'Open': int(Open),
            'Close': int(Close),
        }]

class CollectOpen(beam.DoFn):
    def process(self, element):
        # Returns a list of tuples containing Date and Open value
        result = [(1, element['Open'])]
        print result[0][1]
        return result


def run(argv=None):
    parser = argparse.ArgumentParser()
    parser.add_argument('--input',type=str,
                        help='Input for the pipeline',
                        default='stock_test.csv')
    parser.add_argument('--output',
                        help='Output for the pipeline',
                        default='smit.txt')
    args, pipeline_args = parser.parse_known_args(argv)
    options = PipelineOptions(pipeline_args)
    input_filename = args.input
    output_filename = args.output
    with beam.Pipeline(options=options) as p:
        (p
         | 'ReadInputText' >> beam.io.ReadFromText(args.input,skip_header_lines=1)
         | 'Split' >> beam.ParDo(Split())
         | beam.ParDo(CollectOpen())
         |"Grouping keys Open" >> beam.GroupByKey()
         |"Calculating mean for Open" >> beam.CombineValues(beam.combiners.MeanCombineFn())
         | beam.io.textio.WriteToText(args.output))

if __name__ == '__main__':
    run()
