import argparse
import csv
import logging
import numpy as np
import pandas as pd 
from sklearn import linear_model
import apache_beam as beam
from apache_beam.metrics.metric import Metrics
from apache_beam.options.pipeline_options import PipelineOptions
from apache_beam.options.pipeline_options import SetupOptions
from matplotlib import pyplot as plt

class Predict(beam.DoFn):
    def process(self,element):
        import numpy
        Date,Open,High,Low,Close,volume = element.split(",")
        x_predict = [float(Open),float(High),float(Low)]
        x_axis.append(float(Close))
        x_predict = (numpy.asarray(x_predict)).reshape(1,-1)
        predictions = regr.predict(x_predict)
        y_axis.append(round(predictions[0], 2))
        return [{
            'Date':Date,
            'Open': float(Open),
            'Close': float(Close),
            'Close_predict': predictions[0]
        }]

class CollectOpen(beam.DoFn):
    def process(self, element):
        # Returns a list of tuples containing Date and Open value
        result = [(element['Date'],element['Open'],element['Close'],element['Close_predict'])]
        return result


def run(argv=None):
   # stock = pd.read_csv("F:\\college\\sem 8\\apache-beam-2.4.0\\sdks\\python\\apache_beam\\examples\\complete\\game\stock\\train.csv")  
    import gcsfs
    import matplotlib
    from matplotlib import pyplot as plt
    fs=gcsfs.GCSFileSystem(project='newproj-228110')
    with fs.open('stock_prediction/train.csv') as f:
        stock = pd.read_csv(f)
    global regr
    global x_axis, y_axis
    x_axis = []
    y_axis = []
    regr = linear_model.LinearRegression()
    array = stock.values
    X = array[:,1:4]
    Y = array[:,4]

    regr.fit(X, Y)

    parser = argparse.ArgumentParser()
    parser.add_argument('--input',type=str,
                        help='Input for the pipeline',
                        default='stock_test.csv')
    parser.add_argument('--output',
                        help='Output for the pipeline',
                        default='smit.txt')
    parser.add_argument('--setup_file',
                        help='Output for the pipeline',
                        default='gs://stock_prediction/setup.py')
   
    args, pipeline_args = parser.parse_known_args(argv)
    options = PipelineOptions(pipeline_args)
    options.view_as(SetupOptions).save_main_session = True
    input_filename = args.input
    output_filename = args.output
    with beam.Pipeline(options=options) as p:
        (p
         | 'ReadInputText' >> beam.io.ReadFromText(args.input,skip_header_lines=1)
         | 'Split' >> beam.ParDo(Prediction())
         | beam.ParDo(CollectOpen())
         | beam.io.textio.WriteToText(args.output))
   
    fit = np.polyfit(x_axis,y_axis,1)
    fit_fn = np.poly1d(fit)
    plt.xlabel('Predicted')
    plt.ylabel('Actual')
    plt.title('Stock Prediction')
    plt(x_axis,y_axis, 'yo', x_axis, fit_fn(x_axis), '--k')
    plt.savefig('result_graph_without_gold.png')

if __name__ == '__main__':
    logging.getLogger().setLevel(logging.INFO)
    run()
